@@ -7,8 +7,8 @@
 #' @importFrom jsonlite fromJSON
 NULL
 
-rand_string <- function (length = 10) {
-  paste(sample(c(letters, LETTERS, 0:9), length, replace = TRUE),
+rand_string <- function (length = 10, prefix = NULL) {
+  paste(c(prefix, sample(c(letters, LETTERS, 0:9), length, replace = TRUE)),
         collapse = "")
 }
 
